from osrs.async_api.wiki.prices import (
    AveragePrices,
    ItemMapping,
    LatestPrices,
    TimeSeries,
    WikiPrices,
)

__all__ = ["WikiPrices", "AveragePrices", "LatestPrices", "TimeSeries", "ItemMapping"]



import logging
from enum import Enum

from aiohttp import ClientSession
from pydantic import BaseModel

from osrs.exceptions import PlayerDoesNotExist, Undefined, UnexpectedRedirection
from osrs.utils import RateLimiter

logger = logging.getLogger(__name__)


class Mode(str, Enum):
    OLDSCHOOL = "hiscore_oldschool"
    IRONMAN = "hiscore_oldschool_ironman"
    HARDCORE = "hiscore_oldschool_hardcore_ironman"
    ULTIMATE = "hiscore_oldschool_ultimate"
    DEADMAN = "hiscore_oldschool_deadman"
    SEASONAL = "hiscore_oldschool_seasonal"
    TOURNAMENT = "hiscore_oldschool_tournament"


class Skill(BaseModel):
    id: int
    name: str
    rank: int
    level: int
    xp: int


class Activity(BaseModel):
    id: int
    name: str
    rank: int
    score: int


class PlayerStats(BaseModel):
    skills: list[Skill]
    activities: list[Activity]


class Hiscore:
    BASE_URL = "https://secure.runescape.com"

    def __init__(
        self, proxy: str = "", rate_limiter: RateLimiter = RateLimiter()
    ) -> None:
        self.proxy = proxy
        self.rate_limiter = rate_limiter

    async def get(self, mode: Mode, player: str, session: ClientSession) -> PlayerStats:
        """
        Fetches player stats from the OSRS hiscores API.

        Args:
            mode (Mode): The hiscore mode.
            player (str): The player's username.
            session (ClientSession): The HTTP session.

        Returns:
            PlayerStats: Parsed player statistics.

        Raises:
            UnexpectedRedirection: If a redirection occurs.
            PlayerDoesNotExist: If the player is not found (404 error).
            ClientResponseError: For other HTTP errors.
            Undefined: For anything else that is not a 200
        """
        await self.rate_limiter.check()

        logger.debug(f"Performing hiscores lookup on {player}")
        url = f"{self.BASE_URL}/m={mode.value}/index_lite.json"
        params = {"player": player}

        async with session.get(url, proxy=self.proxy, params=params) as response:
            # when the HS are down it will redirect to the main page.
            # after redirction it will return a 200, so we must check for redirection first
            if response.history and any(r.status == 302 for r in response.history):
                error_msg = (
                    f"Redirection occured: {response.url} - {response.history[0].url}"
                )
                raise UnexpectedRedirection(error_msg)
            elif response.status == 404:
                raise PlayerDoesNotExist(f"player: {player} does not exist.")
            elif response.status != 200:
                # raises ClientResponseError
                response.raise_for_status()
                raise Undefined()
            data = await response.json()
            return PlayerStats(**data)



import pytest
from aiohttp import ClientSession

from osrs.async_api.wiki.prices import (
    AveragePrices,
    Interval,
    ItemMapping,
    LatestPrices,
    TimeSeries,
    WikiPrices,
)


@pytest.mark.asyncio
async def test_get_mapping_valid():
    """Test fetching item mappings successfully."""
    wiki_prices_instance = WikiPrices(
        user_agent="tests - https://github.com/Bot-detector/osrs"
    )
    async with ClientSession() as session:
        mappings = await wiki_prices_instance.get_mapping(session=session)

        assert isinstance(mappings, list), "Mappings should be a list"
        assert len(mappings) > 0, "Mappings list should not be empty"
        assert isinstance(
            mappings[0], ItemMapping
        ), "First mapping should be of type ItemMapping"


@pytest.mark.asyncio
async def test_get_latest_prices_valid():
    """Test fetching the latest prices successfully."""
    wiki_prices_instance = WikiPrices(
        user_agent="tests - https://github.com/Bot-detector/osrs"
    )
    async with ClientSession() as session:
        latest_prices = await wiki_prices_instance.get_latest_prices(session=session)

        assert isinstance(
            latest_prices, LatestPrices
        ), "The returned object is not of type LatestPrices"
        assert latest_prices.data, "Latest prices data should not be empty"


@pytest.mark.asyncio
async def test_get_average_prices_valid():
    """Test fetching average prices with valid parameters."""
    wiki_prices_instance = WikiPrices(
        user_agent="tests - https://github.com/Bot-detector/osrs"
    )
    async with ClientSession() as session:
        average_prices = await wiki_prices_instance.get_average_prices(
            session=session, interval=Interval.FIVE_MIN
        )

        assert isinstance(
            average_prices, AveragePrices
        ), "The returned object is not of type AveragePrices"
        assert average_prices.data, "Average prices data should not be empty"


@pytest.mark.asyncio
async def test_get_average_prices_invalid():
    """Test fetching average prices with an invalid interval."""
    wiki_prices_instance = WikiPrices(
        user_agent="tests - https://github.com/Bot-detector/osrs"
    )
    async with ClientSession() as session:
        with pytest.raises(Exception):
            await wiki_prices_instance.get_average_prices(
                session=session,
                interval="invalid_interval",  # type: ignore
            )


@pytest.mark.asyncio
async def test_get_time_series_valid():
    """Test fetching time series data for a valid item ID and timestep."""
    wiki_prices_instance = WikiPrices(
        user_agent="tests - https://github.com/Bot-detector/osrs"
    )
    async with ClientSession() as session:
        item_id = 4151
        time_series = await wiki_prices_instance.get_time_series(
            session=session, item_id=item_id, timestep=Interval.ONE_HOUR
        )

        assert isinstance(
            time_series, TimeSeries
        ), "The returned object is not of type TimeSeries"
        assert len(time_series.data) > 0, "Time series data should not be empty"


@pytest.mark.asyncio
async def test_get_time_series_invalid():
    """Test fetching time series data for an invalid item ID."""
    wiki_prices_instance = WikiPrices(
        user_agent="tests - https://github.com/Bot-detector/osrs"
    )
    async with ClientSession() as session:
        invalid_item_id = 9999999

        time_series = await wiki_prices_instance.get_time_series(
            session=session, item_id=invalid_item_id, timestep=Interval.ONE_HOUR
        )
        assert isinstance(
            time_series, TimeSeries
        ), "The returned object is not of type TimeSeries"
        assert len(time_series.data) == 0, "Time series data should be empty"



import pytest
from aiohttp import ClientSession

from osrs.async_api.osrs.itemdb import (
    Catalogue,
    Detail,
    Graph,
    Items,
    Mode,
    TradeHistory,
)


@pytest.mark.asyncio
async def test_get_items_valid():
    """Test fetching items with valid parameters"""
    catalogue_instance = Catalogue()
    async with ClientSession() as session:
        items = await catalogue_instance.get_items(
            session=session,
            alpha="a",  # Assume items starting with "A" exist
            page=1,
            mode=Mode.OLDSCHOOL,
            category=1,
        )

        # Assertions to confirm the response is correct
        assert isinstance(items, Items), "The returned object is not of type Items"
        assert items.items, "Items list should not be empty"
        assert items.total > 0, "Total count should be greater than zero"


@pytest.mark.asyncio
async def test_get_items_invalid_page():
    """Test fetching items with an invalid page number"""
    catalogue_instance = Catalogue()
    async with ClientSession() as session:
        items = await catalogue_instance.get_items(
            session=session,
            alpha="A",
            page=9999,  # Assume this page does not exist
            mode=Mode.OLDSCHOOL,
            category=1,
        )

        # Assertions for an empty result or similar handling
        assert isinstance(items, Items), "The returned object is not of type Items"
        assert not items.items, "Items list should be empty for a non-existing page"


@pytest.mark.asyncio
async def test_get_detail_valid():
    """Test fetching details for a valid item ID"""
    catalogue_instance = Catalogue()
    async with ClientSession() as session:
        item_id = 4151  # Assume this is a valid item ID
        item_detail = await catalogue_instance.get_detail(
            session=session, item_id=item_id, mode=Mode.OLDSCHOOL
        )

        # Assertions to confirm the response is correct
        assert isinstance(
            item_detail, Detail
        ), "The returned object is not of type Detail"
        assert item_detail.item.name == "Abyssal whip", "Unexpected item name returned"


@pytest.mark.asyncio
async def test_get_detail_invalid():
    """Test fetching details for an invalid item ID"""
    catalogue_instance = Catalogue()
    async with ClientSession() as session:
        invalid_item_id = 9999999  # Assume this item ID does not exist
        with pytest.raises(
            Exception
        ):  # Replace Exception with a specific exception if defined
            await catalogue_instance.get_detail(
                session=session, item_id=invalid_item_id, mode=Mode.OLDSCHOOL
            )


@pytest.mark.asyncio
async def test_get_graph_valid():
    """Test fetching trade history for a valid item ID"""
    catalogue_instance = Graph()
    async with ClientSession() as session:
        item_id = 4151  # Assume this is a valid item ID
        trade_history = await catalogue_instance.get_graph(
            session=session, item_id=item_id, mode=Mode.OLDSCHOOL
        )

        # Assertions to confirm the response is correct
        assert isinstance(
            trade_history, TradeHistory
        ), "The returned object is not of type TradeHistory"
        assert trade_history.daily, "Daily trade history should not be empty"
        assert trade_history.average, "Average trade history should not be empty"


@pytest.mark.asyncio
async def test_get_graph_invalid():
    """Test fetching trade history for an invalid item ID"""
    catalogue_instance = Graph()
    async with ClientSession() as session:
        invalid_item_id = 9999999  # Assume this item ID does not exist
        with pytest.raises(
            Exception
        ):  # Replace Exception with a specific exception if defined
            await catalogue_instance.get_graph(
                session=session, item_id=invalid_item_id, mode=Mode.OLDSCHOOL
            )



import pytest
from aiohttp import ClientSession

from osrs.async_api.osrs.hiscores import Hiscore, Mode, PlayerStats
from osrs.exceptions import PlayerDoesNotExist


@pytest.mark.asyncio
async def test_get_valid():
    hiscore_instance = Hiscore()
    async with ClientSession() as session:
        player_stats = await hiscore_instance.get(
            mode=Mode.OLDSCHOOL,
            player="extreme4all",
            session=session,
        )

        # Assertions to confirm the response is correct
        assert isinstance(
            player_stats, PlayerStats
        ), "The returned object is not of type PlayerStats"
        assert player_stats.skills, "Skills data should not be empty"
        assert player_stats.activities, "Activities data should not be empty"


@pytest.mark.asyncio
async def test_get_invalid():
    hiscore_instance = Hiscore()
    async with ClientSession() as session:
        with pytest.raises(PlayerDoesNotExist):
            _ = await hiscore_instance.get(
                mode=Mode.OLDSCHOOL,
                player="This_is_not_a_valid_name",
                session=session,
            )



import asyncio
import logging
import time
from collections import deque

logger = logging.getLogger(__name__)


class RateLimiter:
    def __init__(self, calls_per_interval: int = 60, interval: int = 60) -> None:
        self.history = deque(maxlen=calls_per_interval)
        self.interval = interval

    async def check(self):
        """
        Checks if the rate limit is reached and waits if necessary.
        """
        current_time = int(time.time())
        self.history.append(current_time)

        # If history is full, calculate the time difference
        if len(self.history) == self.history.maxlen:
            first_call = self.history[0]
            time_span = current_time - first_call

            # If time_span is less than interval seconds, sleep for the remaining time
            if time_span < self.interval:
                sleep_time = self.interval - time_span
                logger.debug(f"Rate limit reached. Sleeping for {sleep_time} seconds.")
                await asyncio.sleep(sleep_time)



import logging
from enum import Enum

from aiohttp import ClientSession
from pydantic import BaseModel

from osrs.exceptions import Undefined
from osrs.utils import RateLimiter

logger = logging.getLogger(__name__)


class Interval(str, Enum):
    FIVE_MIN = "5m"
    ONE_HOUR = "1h"
    SIX_HOUR = "6h"
    DAY = "24h"


class ItemMapping(BaseModel):
    examine: str
    id: int
    members: bool
    lowalch: int | None = None
    limit: int | None = None
    value: int
    highalch: int | None = None
    icon: str
    name: str


class PriceData(BaseModel):
    high: int | None
    highTime: int | None
    low: int | None
    lowTime: int | None


class LatestPrices(BaseModel):
    data: dict[str, PriceData]


class AveragePriceData(BaseModel):
    avgHighPrice: int | None
    highPriceVolume: int | None
    avgLowPrice: int | None
    lowPriceVolume: int | None


class AveragePrices(BaseModel):
    data: dict[str, AveragePriceData]


class TimeSeriesData(BaseModel):
    timestamp: int
    avgHighPrice: int | None
    avgLowPrice: int | None
    highPriceVolume: int | None
    lowPriceVolume: int | None


class TimeSeries(BaseModel):
    data: list[TimeSeriesData]


class WikiPrices:
    BASE_URL = "https://prices.runescape.wiki/api/v1"

    def __init__(
        self,
        user_agent: str = "",
        proxy: str = "",
        rate_limiter: RateLimiter = RateLimiter(),
    ) -> None:
        """
        Initializes the WikiPrices client for accessing OSRS price data.

        Args:
            user_agent (str): User-Agent like 'price_tracker - @username on Discord'
            proxy (str): Optional proxy URL for requests.
            rate_limiter (RateLimiter): Rate limiter to control request frequency.
        """
        self.proxy = proxy
        self.rate_limiter = rate_limiter

        if user_agent:
            self.user_agent = user_agent
        else:
            inp = input("""
                User-Agent like 'price_tracker - @username on Discord'\n
                user_agent: 
            """)
            if not inp:
                raise Exception("invalid input")
            self.user_agent = inp

    async def fetch_data(self, session: ClientSession, url: str, params: dict = {}):
        """
        Utility method to fetch data from a specific endpoint, with ratelimiter,
        and basic error handling

        Args:
            session (ClientSession): The HTTP session for making the request.
            url (str): The URL endpoint to fetch data from.
            params (Optional[dict]): Query parameters for the request.

        Returns:
            dict: JSON-parsed data from the API response.
        """
        await self.rate_limiter.check()

        async with session.get(url, proxy=self.proxy, params=params) as response:
            if response.status == 400:
                error = await response.json()
                raise Exception(error)
            elif response.status != 200:
                response.raise_for_status()
                raise Undefined("Unexpected error.")
            return await response.json()

    async def get_mapping(self, session: ClientSession):
        """
        Fetches item mappings containing metadata.

        Args:
            session (ClientSession): The HTTP session for making the request.

        Returns:
            List[ItemMapping]: List of ItemMapping objects with metadata for each item.

        Example:
            >>> session = ClientSession()
            >>> wiki_prices = WikiPrices()
            >>> mappings = await wiki_prices.get_mapping(session)
            >>> print(mappings[0].name)  # e.g., '3rd age amulet'
        """
        url = f"{self.BASE_URL}/osrs/mapping"
        data = await self.fetch_data(session=session, url=url)
        return [ItemMapping(**item) for item in data]

    async def get_latest_prices(self, session: ClientSession) -> LatestPrices:
        """
        Fetches the latest prices for all items.

        Args:
            session (ClientSession): The HTTP session for making the request.

        Returns:
            LatestPrices: A dictionary of item IDs to PriceData.

        Example:
            >>> session = ClientSession()
            >>> wiki_prices = WikiPrices()
            >>> latest_prices = await wiki_prices.get_latest_prices(session)
            >>> print(latest_prices.data["2"].high)  # e.g., 240
        """
        url = f"{self.BASE_URL}/osrs/latest"
        data = await self.fetch_data(session=session, url=url)
        return LatestPrices(**data)

    async def get_average_prices(
        self,
        session: ClientSession,
        interval: Interval,
        timestamp: int | None = None,
    ) -> AveragePrices:
        """
        Fetches average prices at a specified interval (5-minute, 1-hour, etc.).

        Args:
            session (ClientSession): The HTTP session for the request.
            interval (Interval): The time interval ('5m', '1h', etc.) for averaging.
            timestamp (Optional[int]): Optional Unix timestamp to retrieve prices for a specific time.

        Returns:
            AveragePrices: A dictionary of item IDs to AveragePriceData.

        Example:
            >>> session = ClientSession()
            >>> wiki_prices = WikiPrices()
            >>> avg_prices = await wiki_prices.get_average_prices(session, Interval.ONE_HOUR)
            >>> print(avg_prices.data["2"].avgHighPrice)  # e.g., 235
        """
        url = f"{self.BASE_URL}/osrs/{interval.value}"
        params = {"timestamp": timestamp} if timestamp else {}
        data = await self.fetch_data(session=session, url=url, params=params)
        return AveragePrices(**data)

    async def get_time_series(
        self, session: ClientSession, item_id: int, timestep: Interval
    ) -> TimeSeries:
        """
        Fetches time-series data for a specific item and timestep.

        Args:
            session (ClientSession): The HTTP session for the request.
            item_id (int): The item ID.
            timestep (Interval): The timestep (e.g., '5m', '1h').

        Returns:
            TimeSeries: A list of TimeSeriesData entries for the specified item and timestep.

        Example:
            >>> session = ClientSession()
            >>> wiki_prices = WikiPrices()
            >>> time_series = await wiki_prices.get_time_series(session, 2, Interval.ONE_HOUR)
            >>> print(time_series.data[0].avgHighPrice)  # e.g., 1310000
        """
        url = f"{self.BASE_URL}/osrs/timeseries"
        params = {"id": item_id, "timestep": timestep.value}
        data = await self.fetch_data(session=session, url=url, params=params)
        return TimeSeries(**data)



import json
import logging
from datetime import datetime
from enum import Enum
from typing import Union

from aiohttp import ClientSession
from pydantic import BaseModel, HttpUrl

from osrs.utils import RateLimiter

logger = logging.getLogger(__name__)


class Mode(str, Enum):
    OLDSCHOOL = "itemdb_oldschool"
    RS3 = "itemdb_rs"


class CurrentPrice(BaseModel):
    trend: str
    price: int | str  # Price can be an int or a formatted string (e.g., "15.5k")


class TodayPrice(BaseModel):
    trend: str
    price: int | str  # Price can also be an int or string (e.g., "+2")


class Item(BaseModel):
    icon: HttpUrl
    icon_large: HttpUrl
    id: int
    type: str
    typeIcon: HttpUrl
    name: str
    description: str
    current: CurrentPrice
    today: TodayPrice
    members: str | bool


class Items(BaseModel):
    total: int
    items: list[Item]


class PriceDetail(BaseModel):
    trend: str
    price: Union[int, str]  # Price can be int or formatted string (e.g., "+9")


class ChangeDetail(BaseModel):
    trend: str
    change: str  # Change is a percentage string (e.g., "+8.0%")


class ItemDetail(BaseModel):
    icon: HttpUrl
    icon_large: HttpUrl
    id: int
    type: str
    typeIcon: HttpUrl
    name: str
    description: str
    current: PriceDetail
    today: PriceDetail
    members: str  # Boolean-like strings ("true"/"false")
    day30: ChangeDetail
    day90: ChangeDetail
    day180: ChangeDetail


class Detail(BaseModel):
    item: ItemDetail


class TradeHistory(BaseModel):
    daily: dict[datetime, int]
    average: dict[datetime, int]


class Catalogue:
    BASE_URL = "https://secure.runescape.com"

    def __init__(
        self, proxy: str = "", rate_limiter: RateLimiter = RateLimiter()
    ) -> None:
        """Initialize the Catalogue with an optional proxy and rate limiter.

        Args:
            proxy (str): Proxy URL to use for API requests. Defaults to "".
            rate_limiter (RateLimiter): Rate limiter to manage request throttling.
                                        Defaults to a new RateLimiter instance.
        """
        self.proxy = proxy
        self.rate_limiter = rate_limiter

    async def get_items(
        self,
        session: ClientSession,
        alpha: str,
        page: int | None = 1,
        mode: Mode = Mode.OLDSCHOOL,
        category: int = 1,
    ) -> Items:
        """Fetch items from the RuneScape item catalog based on alphabetical filter.

        Args:
            session (ClientSession): An active aiohttp session for making requests.
            alpha (str): Alphabetical character to filter item names.
            page (int, optional): Page number to retrieve. Defaults to 1.
            mode (Mode, optional): Game mode (RS3 or OLDSCHOOL). Defaults to Mode.OLDSCHOOL.
            category (int, optional): Category identifier for items. Defaults to 1.

        Returns:
            Items: List of items matching the filter.
        """
        await self.rate_limiter.check()

        url = f"{self.BASE_URL}/m={mode.value}/api/catalogue/items.json"
        params = {"category": category, "alpha": alpha, "page": page}
        params = {k: v for k, v in params.items()}

        logger.debug(f"[GET]: {url=}, {params=}")

        async with session.get(url, proxy=self.proxy, params=params) as response:
            response.raise_for_status()
            data = await response.text()
            return Items(**json.loads(data))

    async def get_detail(
        self,
        session: ClientSession,
        item_id: int,
        mode: Mode = Mode.OLDSCHOOL,
    ) -> Detail:
        """Fetch detailed information about a specific item.

        Args:
            session (ClientSession): An active aiohttp session for making requests.
            item_id (int): Unique identifier for the item.
            mode (Mode, optional): Game mode (RS3 or OLDSCHOOL). Defaults to Mode.OLDSCHOOL.

        Returns:
            Detail: Detailed information about the item.
        """
        await self.rate_limiter.check()

        url = f"{self.BASE_URL}/m={mode.value}/api/catalogue/detail.json"
        params = {"item": item_id}

        logger.debug(f"[GET]: {url=}, {params=}")

        async with session.get(url, proxy=self.proxy, params=params) as response:
            response.raise_for_status()
            data = await response.text()
            return Detail(**json.loads(data))


class Graph:
    BASE_URL = "https://secure.runescape.com"

    def __init__(
        self, proxy: str = "", rate_limiter: RateLimiter = RateLimiter()
    ) -> None:
        """Initialize the Catalogue with an optional proxy and rate limiter.

        Args:
            proxy (str): Proxy URL to use for API requests. Defaults to "".
            rate_limiter (RateLimiter): Rate limiter to manage request throttling.
                                        Defaults to a new RateLimiter instance.
        """
        self.proxy = proxy
        self.rate_limiter = rate_limiter

    async def get_graph(
        self,
        session: ClientSession,
        item_id: int,
        mode: Mode = Mode.OLDSCHOOL,
    ) -> TradeHistory:
        """Fetch trade history graph data for a specific item.

        Args:
            session (ClientSession): An active aiohttp session for making requests.
            item_id (int): Unique identifier for the item.
            mode (Mode, optional): Game mode (RS3 or OLDSCHOOL). Defaults to Mode.OLDSCHOOL.

        Returns:
            TradeHistory: Historical trading data for the item.
        """
        await self.rate_limiter.check()

        url = f"{self.BASE_URL}/m={mode.value}/api/graph/{item_id}.json"

        logger.debug(f"[GET]: {url=}")

        async with session.get(url, proxy=self.proxy) as response:
            response.raise_for_status()
            data = await response.text()
            return TradeHistory(**json.loads(data))
